use Test::More;
use strict;
use warnings;

use_ok $_ for qw(
  Articulate
  Articulate::Syntax
  Articulate::Authentication
  Articulate::Service
  Articulate::Augmentation::Interpreter
  Articulate::Augmentation::Interpreter::Markdown
  Articulate::Error::Forbidden
  Articulate::Error::Unauthorised
  Articulate::Error::BadRequest
  Articulate::Error::AlreadyExists
  Articulate::Error::NotFound
  Articulate::Error::Internal
  Articulate::Item
  Articulate::Sortation
  Articulate::Validation
  Articulate::Construction
  Articulate::Authorisation
  Articulate::Navigation
  Articulate::Storage
  Articulate::Location
  Articulate::Enrichment::DateUpdated
  Articulate::Enrichment::DateCreated
  Articulate::TestEnv
  Articulate::Sortation::String
  Articulate::Sortation::MetaDelver
  Articulate::Sortation::Slug
  Articulate::Sortation::Numeric
  Articulate::Serialisation
  Articulate::Flow::ContentType
  Articulate::Flow::LocationSwitch
  Articulate::Flow::MetaSwitch
  Articulate::Validation::NoScript
  Articulate::FrameworkAdapter
  Articulate::Authentication::Preconfigured
  Articulate::Authentication::Internal
  Articulate::Role::Service
  Articulate::Role::Routes
  Articulate::Role::Sortation::AllYouNeedIsCmp
  Articulate::Role::Flow
  Articulate::Role::Component
  Articulate::Permission
  Articulate::Routes::Login
  Articulate::Routes::TransparentForms
  Articulate::Routes::TransparentPreviews
  Articulate::Routes::Transparent
  Articulate::Serialisation::Asset
  Articulate::Serialisation::TemplateToolkit
  Articulate::Serialisation::SiteConfig
  Articulate::Serialisation::StatusSetter
  Articulate::File
  Articulate::Storage::Local
  Articulate::Storage::DBIC::Simple::Schema
  Articulate::Storage::DBIC::Simple::Schema::Result::Articulate::Item
  Articulate::Storage::DBIC::Simple
  Articulate::Syntax::Routes
  Articulate::Service::List
  Articulate::Service::SimplePreviews
  Articulate::Service::Simple
  Articulate::Service::LoginForm
  Articulate::Service::Login
  Articulate::Service::Error
  Articulate::Service::SimpleForms
  Articulate::Error
  Articulate::Enrichment
  Articulate::LocationSpecification
  Articulate::Response
  Articulate::Request
  Articulate::Augmentation
  Articulate::Item::Article
  Articulate::Construction::LocationBased
  Articulate::Authorisation::OwnerOverride
  Articulate::Authorisation::AlwaysAllow
  Articulate::Authorisation::Preconfigured
);

done_testing;
